/**
* This file contains the core methods in AES:
*   KeyExpansion() takes the user's key and stretches it
*   Cipher() peforms the encryption on a single block
*   InvCipher() undoes the encryption on a single block.
*
* This program is made freely available under the liberal MIT license. 
*  Please see the accompanying file, License.txt, for full details.
**/

/**
* Perform the AES key expansion
**/
void KeyExpansion(byte key[], byte w[]) {
    byte temp[4] = {0};
    int i, j;
    
    memcpy(w, key, 4*Nk);
    #ifdef DEBUG_KEY_EXPANDER
        for(i = 0; i < Nk; i++){
            printf("\nw%d = ", i);
            PrintWord(w + 4*i);
        }
    #endif
    
    i = Nk;
    #ifdef DEBUG_KEY_EXPANDER
        printf("\ni  temp\t\tRotWord\t\tSubWord\t   Rcon[i/Nk]\tXOR\t\tw[i-Nk]\t\tw[i]\n");
    #endif
    while (i < Nb * (Nr+1)) {
        memcpy(temp, (w + 4*(i - 1)), 4*sizeof(byte)); //temp = w[i-1];
        #ifdef DEBUG_KEY_EXPANDER
            printf("%d  ", i);
            PrintWord(temp);
        #endif
        
        if (i % Nk == 0){
            RotWord(temp);
            #ifdef DEBUG_KEY_EXPANDER
                PrintWord(temp);
            #endif
            
            SubWord(temp);
            #ifdef DEBUG_KEY_EXPANDER
                PrintWord(temp);
            #endif
            
            temp[0] ^= Rcon[i/Nk];
            #ifdef DEBUG_KEY_EXPANDER
                printf("%d\t", Rcon[i/Nk]);
                PrintWord(temp);
            #endif
        }
        else if (Nk > 6 && i % Nk == 4){
            SubWord(temp);
            #ifdef DEBUG_KEY_EXPANDER
                PrintWord(temp);
            #endif
        }
        #ifdef DEBUG_KEY_EXPANDER
            else
                printf("\t\t\t\t\t\t\t");
            // w[i-Nk]
            PrintWord(w + 4*(i - Nk));
        #endif
        
        //w[i] = w[i-Nk] ^ temp
        for(j = 0; j < 4; j++)
            w[4*i + j] = w[4*(i-Nk)+j] ^ temp[j];
        #ifdef DEBUG_KEY_EXPANDER
            PrintWord(w + (i*4));
            printf("\n");
        #endif
        
        #ifdef DEBUG_KEY_EXPANDER
        for(j = 0; j < 8; j++){
            printf("\nw%d = ", j);
            PrintWord(w + 4*j);
        }
        #endif
        
        i++;
    }
}


/**
*  Encrypt one block (128 bits)
**/
void Cipher(byte in[], byte out[], byte w[])
{
    int round = 0;
    byte state[BLOCK_SIZE];
    
    memcpy(state, in, sizeof(state));  // state[] = in[]

    #if VERBOSE >= 3
        printf("round[%2d].input\t\t", round);
        PrintState(state);
    #endif
    
    #if VERBOSE >= 3
        printf("round[%2d].k_sch\t\t", round);
        PrintState(w);
    #endif
    AddRoundKey(state, w);
    
    for(round = 1; round < Nr; round++)
    {
        #if VERBOSE >= 3
            printf("round[%2d].start\t\t", round);
            PrintState(state);
        #endif
    
        SubBytes(state);
        #if VERBOSE >= 3
            printf("round[%2d].s_box\t\t", round);
            PrintState(state);
        #endif
        
        ShiftRows(state);
        #if VERBOSE >= 3
            printf("round[%2d].s_row\t\t", round);
            PrintState(state);
        #endif
        
        MixColumns(state);
        #if VERBOSE >= 3
            printf("round[%2d].m_col\t\t", round);
            PrintState(state);
        #endif
        
        AddRoundKey(state, (w + round*BLOCK_SIZE));
        #if VERBOSE >= 3
            printf("round[%2d].k_sch\t\t", round);
            PrintState((w + round*BLOCK_SIZE));
        #endif
    }
    
    #if VERBOSE >= 3
        printf("round[%2d].start\t\t", round);
        PrintState(state);
    #endif
        
    SubBytes(state);
    #if VERBOSE >= 3
        printf("round[%2d].s_box\t\t", round);
        PrintState(state);
    #endif
    
    ShiftRows(state);
    #if VERBOSE >= 3
        printf("round[%2d].s_row\t\t", round);
        PrintState(state);
    #endif
    
    AddRoundKey(state, (w + Nr*BLOCK_SIZE)); 
    #if VERBOSE >= 3
        printf("round[%2d].k_sch\t\t", round);
        PrintState((w + Nr*BLOCK_SIZE));
    #endif
    
    memcpy(out, state, BLOCK_SIZE);  // out[] = state[]
    #if VERBOSE >= 3
        printf("round[%2d].output\t", round);
        PrintState(out);
    #endif
}

/**
*  Decrypt one block (128 bits)
**/
void InvCipher(byte in[], byte out[], byte w[])
{
    int round = 0;
    byte state[BLOCK_SIZE];
    
    memcpy(state, in, BLOCK_SIZE);  // state[] = in[]

    #if VERBOSE >= 3
        printf("round[%2d].iinput\t", round);
        PrintState(state);
    #endif
    
    AddRoundKey(state, (w + Nr*BLOCK_SIZE));
    #if VERBOSE >= 3
        printf("round[%2d].ik_sch\t", round);
        PrintState(state);
    #endif
    
    for(round = Nr - 1; round >= 1; round--)
    {
        #if VERBOSE >= 3
            printf("round[%2d].istart\t", round);
            PrintState(state);
        #endif
    
        InvShiftRows(state);
        #if VERBOSE >= 3
            printf("round[%2d].is_row\t", round);
            PrintState(state);
        #endif
        
        InvSubBytes(state);
        #if VERBOSE >= 3
            printf("round[%2d].is_box\t", round);
            PrintState(state);
        #endif
        
        AddRoundKey(state, (w + round*BLOCK_SIZE));
        #if VERBOSE >= 3
            printf("round[%2d].ik_sch\t", round);
            PrintState(state);
        #endif
        
        InvMixColumns(state);
        #if VERBOSE >= 3
            printf("round[%2d].im_col\t", round);
            PrintState(state);
        #endif
    }
    
    InvShiftRows(state);
    #if VERBOSE >= 3
        printf("round[%2d].is_box\t", round);
        PrintState(state);
    #endif
    
    InvSubBytes(state);
    #if VERBOSE >= 3
        printf("round[%2d].is_box\t", round);
        PrintState(state);
    #endif
    
    AddRoundKey(state, w);
    #if VERBOSE >= 3
        printf("round[%2d].ik_sch\t", round);
        PrintState(state);
    #endif

    memcpy(out, state, BLOCK_SIZE);  // out[] = state[]
    #if VERBOSE >= 3
        printf("round[%2d].ioutput\t", round);
        PrintState(out);
    #endif
}
