/**
* ProjectAES is a command-line implementation of the Advanced Encryption Standard (AES) algorithm.
*
* This project is based on the details & examples provided at:
*       http://csrc.nist.gov/groups/ST/toolkit/block_ciphers.html
*       http://en.wikipedia.org/wiki/Advanced_Encryption_Standard
*       http://www.progressive-coding.com/tutorial.php?id=0
*
* This program is made freely available under the liberal MIT license. 
*  Please see the accompanying file, License.txt, for full details.
*/


/**
* Debuggin method to print the contents of the state array
**/
void PrintState(byte state[]){
    int i;
    for(i = 0; i < BLOCK_SIZE; i++)
        printf("%02x", state[i]);
    printf("\n");
}

/**
* Debuggin method to print the contents of the state array
**/
void PrintKey(byte key[]){
    int i;
    for(i = 0; i < 4*Nk; i++)
        printf("%02x", key[i]);
    printf("\n");
}

/**
* Debugging method to print a word
**/
void PrintWord(byte w[]) {
    int i;
    for(i = 0; i < 4; i++)
        printf("%02x", w[i]);
        
    printf("\t");
}

void UT_subbytes() {
    int pass = 0;
    
    printf("SubBytes");
    
    // Test AES128
    currentMode = AES128;
    byte state[] = {0x00,0x10,0x20,0x30,0x40,0x50,0x60,0x70,0x80,0x90,0xa0,0xb0,0xc0,0xd0,0xe0,0xf0};
    byte expected[] = {0x63, 0xca, 0xb7, 0x04, 0x09, 0x53, 0xd0, 0x51, 0xcd, 0x60, 0xe0, 0xe7, 0xba, 0x70, 0xe1, 0x8c};
    
    SubBytes(state);
    
    if(memcmp(state, expected, sizeof(expected)) == 0)
        pass = 1;
    
    if(pass)
        printf("...Pass!\n");
    else
        printf("...Fail!\n");
}

void UT_shiftrows() {
    int pass = 0;
    
    printf("ShiftRows");
    
    // Test AES128
    currentMode = AES128;
    byte state[] = {0x63, 0xca, 0xb7, 0x04, 0x09, 0x53, 0xd0, 0x51, 0xcd, 0x60, 0xe0, 0xe7, 0xba, 0x70, 0xe1, 0x8c};
    byte expected[] = {0x63, 0x53, 0xe0, 0x8c, 0x09, 0x60, 0xe1, 0x04, 0xcd, 0x70, 0xb7, 0x51, 0xba, 0xca, 0xd0, 0xe7};
    
    ShiftRows(state);
    
    if(memcmp(state, expected, sizeof(expected)) == 0)
        pass = 1;
    
    if(pass)
        printf("...Pass!\n");
    else
        printf("...Fail!\n");
}

void UT_addroundkey() {
    int pass = 0;
    
    printf("AddRoundKey");
    
    // Test AES128
    currentMode = AES128;
    byte state[] = {0x04, 0x66, 0x81, 0xe5, 0xe0, 0xcb, 0x19, 0x9a, 0x48, 0xf8, 0xd3, 0x7a, 0x28, 0x06, 0x26, 0x4c};
    byte key[] = {0xa0, 0xfa, 0xfe, 0x17, 0x88, 0x54, 0x2c, 0xb1, 0x23, 0xa3, 0x39, 0x39, 0x2a, 0x6c, 0x76, 0x05};
    byte expected[] = {0xa4, 0x9c, 0x7f, 0xf2, 0x68, 0x9f, 0x35, 0x2b, 0x6b, 0x5b, 0xea, 0x43, 0x02, 0x6a, 0x50, 0x49};
    
    AddRoundKey(state, key);
    
    if(memcmp(state, expected, sizeof(expected)) == 0)
        pass = 1;
    
    if(pass)
        printf("...Pass!\n");
    else
        printf("...Fail!\n");
}

void UT_invsubbytes() {
    int pass = 0;
    
    printf("InvSubBytes");
    
    // Test AES128
    currentMode = AES128;
    byte expected[] = {0x00,0x10,0x20,0x30,0x40,0x50,0x60,0x70,0x80,0x90,0xa0,0xb0,0xc0,0xd0,0xe0,0xf0};
    byte state[] = {0x63, 0xca, 0xb7, 0x04, 0x09, 0x53, 0xd0, 0x51, 0xcd, 0x60, 0xe0, 0xe7, 0xba, 0x70, 0xe1, 0x8c};
    
    InvSubBytes(state);
    
    if(memcmp(state, expected, sizeof(expected)) == 0)
        pass = 1;
    
    if(pass)
        printf("...Pass!\n");
    else
        printf("...Fail!\n");
}

void UT_invshiftrows() {
    int pass = 0;
    
    printf("InvShiftRows");
    
    // Test AES128
    currentMode = AES128;
    byte state[] = {0x63, 0x53, 0xe0, 0x8c, 0x09, 0x60, 0xe1, 0x04, 0xcd, 0x70, 0xb7, 0x51, 0xba, 0xca, 0xd0, 0xe7};
    byte expected[] = {0x63, 0xca, 0xb7, 0x04, 0x09, 0x53, 0xd0, 0x51, 0xcd, 0x60, 0xe0, 0xe7, 0xba, 0x70, 0xe1, 0x8c};
    
    InvShiftRows(state);
    
    if(memcmp(state, expected, sizeof(expected)) == 0)
        pass = 1;
    
    if(pass)
        printf("...Pass!\n");
    else
        printf("...Fail!\n");
}

void UT_mixcolumns() {
    int pass = 0;
    
    printf("MixColumns");
    
    // Test AES128
    currentMode = AES128;
    byte state[] = {0x63, 0x53, 0xe0, 0x8c, 0x09, 0x60, 0xe1, 0x04, 0xcd, 0x70, 0xb7, 0x51, 0xba, 0xca, 0xd0, 0xe7};
    byte expected[] = {0x5f, 0x72, 0x64, 0x15, 0x57, 0xf5, 0xbc, 0x92, 0xf7, 0xbe, 0x3b, 0x29, 0x1d, 0xb9, 0xf9, 0x1a};
    
    MixColumns(state);
    
    if(memcmp(state, expected, sizeof(expected)) == 0)
        pass = 1;
    
    if(pass)
        printf("...Pass!\n");
    else
        printf("...Fail!\n");
}
void UT_rotword() {
    int pass = 0;
    
    printf("RotWord");
    
    // Test AES128
    currentMode = AES128;
    byte state[] = {0x09, 0xcf, 0x4f, 0x3c};
    byte expected[] = {0xcf, 0x4f, 0x3c, 0x09};
    
    RotWord(state);
    
    if(memcmp(state, expected, sizeof(expected)) == 0)
        pass = 1;
    
    if(pass)
        printf("...Pass!\n");
    else
        printf("...Fail!\n");
}

void UT_keyexpansion() {
    int pass = 0;
    
    printf("KeyExpansion");
    
    // Test AES128
    currentMode = AES128;
    byte key[] = {0x2b, 0x7e, 0x15, 0x16, 0x28, 0xae, 0xd2, 0xa6, 0xab, 0xf7, 0x15, 0x88, 0x09, 0xcf, 0x4f, 0x3c};
    byte w[BLOCK_SIZE*(Nr+1)];
    byte expected[] = {0xb6, 0x63, 0x0c, 0xa6};
    
    KeyExpansion(key, w);
    
    if(memcmp(w + NUM_BYTES_IN_WORD *(Nb*(Nr+1) - 1), expected, sizeof(expected)) == 0)
        pass = 1;
    
    // Test AES192
    currentMode = AES192;
    byte key2[] = {0x8e, 0x73, 0xb0, 0xf7, 0xda, 0x0e, 0x64, 0x52, 0xc8, 0x10, 0xf3, 0x2b, 0x80, 0x90, 0x79, 0xe5, 0x62, 0xf8, 0xea, 0xd2, 0x52, 0x2c, 0x6b, 0x7b};
    byte w2[BLOCK_SIZE*(Nr+1)];
    byte expected2[] = {0x01, 0x00, 0x22, 0x02};
    
    KeyExpansion(key2, w2);
    
    if(memcmp(w2 + NUM_BYTES_IN_WORD*(Nb*(Nr+1) - 1), expected2, sizeof(expected2)) == 0)
        pass = 1;
    
    
    // Test AES256
    currentMode = AES256;
    byte key3[] = {0x60, 0x3d, 0xeb, 0x10, 0x15, 0xca, 0x71, 0xbe, 0x2b, 0x73, 0xae, 0xf0, 0x85, 0x7d, 0x77, 0x81, 0x1f, 0x35, 0x2c, 0x07, 0x3b, 0x61, 0x08, 0xd7, 0x2d, 0x98, 0x10, 0xa3, 0x09, 0x14, 0xdf, 0xf4};
    byte w3[BLOCK_SIZE*(Nr+1)];
    byte expected3[] = {0x70, 0x6c, 0x63, 0x1e};
    
    KeyExpansion(key3, w3);
    
    if(memcmp(w3 + NUM_BYTES_IN_WORD*(Nb*(Nr+1) - 1), expected3, sizeof(expected3)) == 0)
        pass = 1;
    
    if(pass)
        printf("...Pass!\n");
    else
        printf("...Fail!\n");
}

void RunUnitTests() {
    printf("Unit Tests:\n");
    UT_addroundkey();
    
    UT_subbytes();
    UT_invsubbytes();
    
    UT_shiftrows();
    UT_invshiftrows();
    
    UT_mixcolumns();
    
    UT_rotword();
    UT_keyexpansion();
}
